@@ -38,7 +38,6 @@ class Node(object):
 
     def line_repr(self, depth, **kwargs):
         """Control how node is displayed in tree representation.
-
         _
         ├── one                                           end
         │   └── two                                     myEnd
@@ -52,9 +51,6 @@ class Node(object):
             return "{}", ""
         return "[]", ""
 
-    def clone(self, deep=False):
-        return copy.deepcopy(self) if deep else copy.copy(self)
-
     def __eq__(self, other):
         if not isinstance(other, self.__class__):
             return False
